/*
 * @(#)IKApplication.cpp	1.00 10th June 2000
 * 
 * Contains functions to implement the sample application that IKTool
 * demonstrates.
 *
 * Modification Log:
 */

/**
 *
 * IKTool is a sample application using iKey, an user identity token from
 * Rainbow Technologies (http://www.rainbow.com).
 * Using this application you can personalize an iKey, and store data either
 * protected with a pin number or without protection, as you desire.
 * 
 * Several generic functions have been written for iKey, so that they
 * can be used with any type of application. 
 * 
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 10th June, 2000
 */


#include <string.h>
#include <stdio.h>
#include "IKAuthTool.hxx"
#include "IKFileTool.hxx"
#include "IKConnectTool.hxx"
#include "IKApplication.hxx"

#define APP_DATA_FILE_ID	0x00000001
#define APP_HMAC1_FILE_ID	0x00000002
#define APP_HMAC2_FILE_ID	0x00000003

#define APP_DATA_FILE_SIZE	MAX_IKEY_USER_INFO_SIZE
#define APP_HMAC_FILE_SIZE 16

#define PLAIN_APP_DIR_ID	0x00007f00
#define PIN_APP_DIR_ID	0x00007f30

#define HMAC_SECRET "IKApplication"


#ifdef _DEBUG
#define DEBUG_PRINT(x) printf("%s", x);
#define WAIT_FOR_ENTER { printf("Press Enter Key..."); fflush(stdin); fflush(stdout); getchar(); }
#else
#define DEBUG_PRINT(x)
#define WAIT_FOR_ENTER 
#endif

static int CreateAndWriteHMACFiles(PRNBTKN_CONTEXT pContext, int iWithPin, char *sMsg);
static int CreateAppDir(PRNBTKN_CONTEXT pContext, int iWithPin, char *sMsg);
static int CreateAndWriteUserFile(PRNBTKN_CONTEXT pContext, int iWithPin, unsigned char *sData, unsigned char DataLen, char *sMsg);
static int WriteIKeyUserFile(PRNBTKN_CONTEXT pContext, int iWithPin, unsigned char *sData, unsigned char DataLen, char *sMsg);
static int VerifyHMACFiles(PRNBTKN_CONTEXT pContext, int iWithPin, char *sMsg);
static int ReadIKeyUserFile(PRNBTKN_CONTEXT pContext, int iWithPin, unsigned char *sData, unsigned char DataLen, char *sMsg);
static int AccessIKey(int iMode, int iWithPin, unsigned char *sData, unsigned char DataLen, unsigned char *Pin, char *sMsg);

int __stdcall AccessPinUserIKey(int iMode, unsigned char *sData, unsigned char DataLen, unsigned char *Pin, char *sMsg)
{
	return AccessIKey(iMode, 1, sData, DataLen, Pin, sMsg);
}

int __stdcall AccessPlainUserIKey(int iMode, unsigned char *sData, unsigned char DataLen, char *sMsg)
{
	return AccessIKey(iMode, 0, sData, DataLen, 0, sMsg);
}

int __stdcall UnblockPinUserIKey(unsigned char *Pin, char *sMsg)
{
	RNBTKN_CONTEXT stContext;
	unsigned long lSysErr;
	int iRet = -1;
	unsigned long lPin;

	if (0 != ConnectIKey(&stContext, &lSysErr, sMsg))
	{
		return -1;
	}

	lPin = 0;
	sscanf((const char *)Pin, "%lx", &lPin);
	if(0 != VerifyIKeyPIN(&stContext, lPin, &lSysErr, sMsg))
	{
		goto DISCONNECT_AND_EXIT;
	}

	iRet = 0;

DISCONNECT_AND_EXIT:
	if (0 != DisconnectIKey(&stContext, &lSysErr, sMsg))
	{
		return -1;
	}
	return iRet;
}

int __stdcall ChangePinInUserIKey(unsigned char *OldPin, unsigned char *NewPin, char *sMsg)
{
	RNBTKN_CONTEXT stContext;
	unsigned long lSysErr;
	int iRet = -1;
	unsigned long lOldPin, lPin;

	if (0 != ConnectIKey(&stContext, &lSysErr, sMsg))
	{
		return -1;
	}

	lPin = 0;
	sscanf((const char *)NewPin, "%lx", &lPin);
	lOldPin = 0;
	sscanf((const char *)OldPin, "%lx", &lOldPin);
	
	if(0 != ModifyIKeyPIN(&stContext, lOldPin, lPin, &lSysErr, sMsg))
	{
		goto DISCONNECT_AND_EXIT;
	}
	iRet = 0;

DISCONNECT_AND_EXIT:
	if (0 != DisconnectIKey(&stContext, &lSysErr, sMsg))
	{
		return -1;
	}
	return iRet;
}

static int AccessIKey(int iMode, int iWithPin, unsigned char *sData, unsigned char DataLen, unsigned char *Pin, char *sMsg)
{
	RNBTKN_CONTEXT stContext;
	unsigned long lSysErr, lPin;
	int iRet = -1;

	if (0 != ConnectIKey(&stContext, &lSysErr, sMsg))
	{
		return -1;
	}
	DEBUG_PRINT("Connected\n");
	WAIT_FOR_ENTER;

	if (CREATE_MODE == iMode)
	{
		DEBUG_PRINT("Create mode\n");
		WAIT_FOR_ENTER;
		// verify master key and create app dir
		if (0 != CreateAppDir(&stContext, iWithPin, sMsg))
		{
			goto DISCONNECT_AND_EXIT;
		}
		DEBUG_PRINT("Created app dir\n");
		WAIT_FOR_ENTER;

		// create and write HMAC files
		if(0 != CreateAndWriteHMACFiles(&stContext, iWithPin, sMsg))
		{
			goto DISCONNECT_AND_EXIT;
		}
		DEBUG_PRINT("Created and wrote HMAC files\n");
		WAIT_FOR_ENTER;

		if(iWithPin)
		{
			// verify pin
			lPin = 0;
			sscanf("12345678", "%lx", &lPin);
			if(0 != VerifyIKeyPIN(&stContext, lPin, &lSysErr, sMsg))
			{
				goto DISCONNECT_AND_EXIT;
			}
			DEBUG_PRINT("Verified pin\n");
			WAIT_FOR_ENTER;		
		}

		// create app file
		if(0 != CreateAndWriteUserFile(&stContext, iWithPin, sData, DataLen, sMsg))
		{
			goto DISCONNECT_AND_EXIT;
		}

		DEBUG_PRINT("Created and wrote user files\n");
		WAIT_FOR_ENTER;

		// set pin
		if(iWithPin)
		{
			unsigned long lOldPin;

			DEBUG_PRINT("With Pin\n");
			WAIT_FOR_ENTER;

			lPin = 0;
			sscanf((const char *)Pin, "%lx", &lPin);
			lOldPin = 0;
			sscanf("12345678", "%lx", &lOldPin);
			
			if(0 != ModifyIKeyPIN(&stContext, lOldPin, lPin, &lSysErr, sMsg))
			{
				goto DISCONNECT_AND_EXIT;
			}
			DEBUG_PRINT("Modified Pin\n");
			WAIT_FOR_ENTER;
		}
	}
	else
	{
		unsigned long lAppDF;
		// select app df
		lAppDF = iWithPin ? PIN_APP_DIR_ID : PLAIN_APP_DIR_ID;

		DEBUG_PRINT("Read or write mode\n");
		WAIT_FOR_ENTER;
		
		if(0 != OpenIKeyFile(&stContext, lAppDF, APP_DATA_FILE_ID, &lSysErr, sMsg))
		{
			goto DISCONNECT_AND_EXIT;
		}
		DEBUG_PRINT("Opened file\n");
		WAIT_FOR_ENTER;		

		if(iWithPin)
		{
			DEBUG_PRINT("With pin\n");
			WAIT_FOR_ENTER;		
			// verify master key
			if(0 != VerifyIKeyMasterKey(&stContext, &lSysErr, sMsg))
			{
				goto DISCONNECT_AND_EXIT;
			}
			DEBUG_PRINT("Verified master key\n");
			WAIT_FOR_ENTER;		
			// verify pin
			lPin = 0;
			sscanf((const char *)Pin, "%lx", &lPin);
			if(0 != VerifyIKeyPIN(&stContext, lPin, &lSysErr, sMsg))
			{
				goto DISCONNECT_AND_EXIT;
			}
			DEBUG_PRINT("Verified pin\n");
			WAIT_FOR_ENTER;
		}

		// verify hmac
		if(0 != VerifyHMACFiles(&stContext, iWithPin, sMsg))
		{
			goto DISCONNECT_AND_EXIT;
		}
		DEBUG_PRINT("Verified HMAC\n");
		WAIT_FOR_ENTER;		

		if(READ_MODE == iMode)
		{
			DEBUG_PRINT("Read mode\n");
			WAIT_FOR_ENTER;		
			// read app file
			if(0 != ReadIKeyUserFile(&stContext, iWithPin, sData, DataLen, sMsg))
			{
				goto DISCONNECT_AND_EXIT;
			}
			DEBUG_PRINT("Read file\n");
			WAIT_FOR_ENTER;
		}
		else
		{
			DEBUG_PRINT("Write mode\n");
			WAIT_FOR_ENTER;		
			// write app file
			if(0 != WriteIKeyUserFile(&stContext, iWithPin, sData, DataLen, sMsg))
			{
				goto DISCONNECT_AND_EXIT;
			}
			DEBUG_PRINT("Wrote User file.\n");
			WAIT_FOR_ENTER;
		}
	}
	
	iRet = 0;

DISCONNECT_AND_EXIT:
	if (0 != DisconnectIKey(&stContext, &lSysErr, sMsg))
	{
		return -1;
	}
	DEBUG_PRINT("Disconnected\n");
	WAIT_FOR_ENTER;		
	return iRet;
}


static int ReadIKeyUserFile(PRNBTKN_CONTEXT pContext, int iWithPin, unsigned char *sData, unsigned char DataLen, char *sMsg)
{
	unsigned long lUserDataLen;
	unsigned long lAppDirID;
	unsigned long lSysErr;

	lAppDirID = iWithPin ? PIN_APP_DIR_ID : PLAIN_APP_DIR_ID;

	// open app file
	if(0 != OpenIKeyFile(pContext, lAppDirID, APP_DATA_FILE_ID, &lSysErr, sMsg))
	{
		return -1;
	}
	DEBUG_PRINT("Opened file\n");
	WAIT_FOR_ENTER;

	// read app file
	lUserDataLen = DataLen;
	if(0 != ReadIKeyFile(pContext, sData, &lUserDataLen, &lSysErr, sMsg))
	{
		return -1;
	}
	if(DataLen < lUserDataLen)
	{
		sprintf(sMsg, "Error reading %d bytes from user file. Read %d bytes.", DataLen, lUserDataLen);
		return -1;
	}
	DEBUG_PRINT("Read file\n");
	WAIT_FOR_ENTER;

	return 0;
}

static int CreateAndWriteUserFile(PRNBTKN_CONTEXT pContext, int iWithPin, unsigned char *sData, unsigned char DataLen, char *sMsg)
{
	unsigned char cReadAccess, cWriteAccess;
	unsigned long lAppDirID;
	unsigned long lSysErr;

	lAppDirID = iWithPin ? PIN_APP_DIR_ID : PLAIN_APP_DIR_ID;

	if(iWithPin)
	{
		cWriteAccess = cReadAccess = RNBTKN_ACCESS_PIN;
	}
	else
	{
		cWriteAccess = cReadAccess = RNBTKN_ACCESS_ALWAYS;
	}

	// create app file
	if(0 != CreateIKeyFile(pContext, lAppDirID, APP_DATA_FILE_ID, APP_DATA_FILE_SIZE, 
			   RNBTKN_FILETYPE_DATA, 
			   cReadAccess, cWriteAccess, RNBTKN_ACCESS_NEVER,
			   &lSysErr, sMsg))
	{
		return -1;
	}
	DEBUG_PRINT("Created User file.\n");
	WAIT_FOR_ENTER;		

	// write app file
	if(0 != WriteIKeyUserFile(pContext, iWithPin, sData, DataLen, sMsg))
	{
		return -1;
	}

	return 0;
}

static int WriteIKeyUserFile(PRNBTKN_CONTEXT pContext, int iWithPin, unsigned char *sData, unsigned char DataLen, char *sMsg)
{
	unsigned long lUserDataLen;
	unsigned long lAppDirID;
	unsigned long lSysErr;

	lAppDirID = iWithPin ? PIN_APP_DIR_ID : PLAIN_APP_DIR_ID;

	// write app file
	if(0 != OpenIKeyFile(pContext, lAppDirID, APP_DATA_FILE_ID, &lSysErr, sMsg))
	{
		return -1;
	}
	DEBUG_PRINT("Opened User file.\n");
	WAIT_FOR_ENTER;		

	lUserDataLen = DataLen;
	if(0 != WriteIKeyFile(pContext, sData, &lUserDataLen, &lSysErr, sMsg))
	{
		return -1;
	}
	if(DataLen > lUserDataLen)
	{
		sprintf(sMsg, "Error writing %d bytes to user file. Wrote %d bytes.", DataLen, lUserDataLen);
		return -1;
	}
	DEBUG_PRINT("Wrote User file.\n");
	WAIT_FOR_ENTER;		

	return 0;
}

static int VerifyHMACFiles(PRNBTKN_CONTEXT pContext, int iWithPin, char *sMsg)
{
	unsigned long lAppDirID;
	unsigned long lSysErr;

	lAppDirID = iWithPin ? PIN_APP_DIR_ID : PLAIN_APP_DIR_ID;

	if(0 != VerifyIKeyHMACKeys(pContext, lAppDirID, APP_HMAC1_FILE_ID, APP_HMAC2_FILE_ID,
				   (unsigned char *)HMAC_SECRET, strlen(HMAC_SECRET), &lSysErr, sMsg))
	{
		return -1;
	}
	return 0;
}

static int CreateAndWriteHMACFiles(PRNBTKN_CONTEXT pContext, int iWithPin, char *sMsg)
{
	unsigned long lSysErr;
	unsigned char sKey1[16], sKey2[16];
	unsigned long iKeyLen;
	unsigned long lAppDirID;

	lAppDirID = iWithPin ? PIN_APP_DIR_ID : PLAIN_APP_DIR_ID;

	// create hmac keys
	if(0 != CreateIKeyHMACKeys(sKey1, sKey2, (unsigned char *)HMAC_SECRET, strlen(HMAC_SECRET)))
	{
		sprintf(sMsg, "%s", "Error creating HMAC keys.");
		return -1;
	}

	// create hmac files
	if(0 != CreateIKeyFile(pContext, lAppDirID, APP_HMAC1_FILE_ID, APP_HMAC_FILE_SIZE, 
			   RNBTKN_FILETYPE_KEY, 
			   RNBTKN_ACCESS_NEVER, // read access
			   RNBTKN_ACCESS_MASTERKEY, // write access
			   RNBTKN_ACCESS_ALWAYS, // crypt access
			   &lSysErr, sMsg))
	{
		return -1;
	}

	if(0 != CreateIKeyFile(pContext, lAppDirID, APP_HMAC2_FILE_ID, APP_HMAC_FILE_SIZE, 
			   RNBTKN_FILETYPE_KEY, 
			   RNBTKN_ACCESS_NEVER, // read access
			   RNBTKN_ACCESS_MASTERKEY, // write access
			   RNBTKN_ACCESS_ALWAYS, // crypt access
			   &lSysErr, sMsg))
	{
		return -1;
	}

	// write hmac files
	iKeyLen = 16;
	if(0 != OpenIKeyFile(pContext, lAppDirID, APP_HMAC1_FILE_ID, &lSysErr, sMsg))
	{
		return -1;
	}

	if(0 != WriteIKeyFile(pContext, sKey1, &iKeyLen, &lSysErr, sMsg))
	{
		return -1;
	}

	if(16 != iKeyLen)
	{
		sprintf(sMsg, "Error writing 16 bytes to key1. Wrote %d bytes.", iKeyLen);
		return -1;
	}

	iKeyLen = 16;
	if(0 != OpenIKeyFile(pContext, lAppDirID, APP_HMAC2_FILE_ID, &lSysErr, sMsg))
	{
		return -1;
	}

	if(0 != WriteIKeyFile(pContext, sKey2, &iKeyLen, &lSysErr, sMsg))
	{
		return -1;
	}

	if(16 != iKeyLen)
	{
		sprintf(sMsg, "Error writing 16 bytes to key2. Wrote %d bytes.", iKeyLen);
		return -1;
	}

	return 0;
}

static int CreateAppDir(PRNBTKN_CONTEXT pContext, int iWithPin, char *sMsg)
{
	unsigned long lSysErr;
	unsigned long lAppDirID;

	lAppDirID = iWithPin ? PIN_APP_DIR_ID : PLAIN_APP_DIR_ID;

	// verify master key
	if (0 != VerifyIKeyMasterKey(pContext, &lSysErr, sMsg))
	{
		return -1;
	}

	// create app df
	if (0 != CreateIKeyDir(pContext, lAppDirID, DIRECTORY_SIZE, &lSysErr, sMsg))
	{
		return -1;
	}
	return 0;
}

