/*
 * @(#)IKFileTool.cpp	1.00 10th June 2000
 * 
 * Functions that take care of file manipulations on an iKey.
 *
 * Modification Log:
 */

/**
 *
 * IKTool is a sample application using iKey, an user identity token from
 * Rainbow Technologies (http://www.rainbow.com).
 * Using this application you can personalize an iKey, and store data either
 * protected with a pin number or without protection, as you desire.
 * 
 * Several generic functions have been written for iKey, so that they
 * can be used with any type of application. 
 * 
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 10th June, 2000
 */


#include <stdio.h>
#include <string.h>
#include "IKFileTool.hxx"
#include "IKMessageTool.hxx"

int OpenIKeyFile(PRNBTKN_CONTEXT pContext, unsigned long lDir, unsigned long lFile, unsigned long *plSysErr, char *sMsg)
{
	*plSysErr = RnbTkn_SelectFile(pContext, lDir, lFile);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}
	return 0;
}

void GetOpenFileInfo(PRNBTKN_CONTEXT pContext, 
					unsigned long *plFileSize, 
					int *piReadAccess,
					int *piWriteAccess,
					int *piCryptAccess,
					unsigned char *pcFileType)
{
	if(plFileSize) *plFileSize = pContext->FileSize;
	if(piReadAccess) *piReadAccess = (pContext->FileAccess & RNBTKN_ACCESS_READ);
	if(piWriteAccess) *piWriteAccess = (pContext->FileAccess & RNBTKN_ACCESS_WRITE);
	if(piCryptAccess) *piCryptAccess = (pContext->FileAccess & RNBTKN_ACCESS_CRYPT);
	if(pcFileType) *pcFileType = pContext->FileType;
}

int CloseIKeyFile(PRNBTKN_CONTEXT pContext, unsigned long *plSysErr, char *sMsg)
{
	*plSysErr = RnbTkn_CloseFile(pContext);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}
	return 0;
}

int ReadIKeyFile(PRNBTKN_CONTEXT pContext, unsigned char *sBuff, unsigned long *plLen, unsigned long *plSysErr, char *sMsg)
{
	unsigned long lActLen;
	unsigned char sAlignBuff[256];
	memset(sAlignBuff, 0, 256);
	memcpy(sAlignBuff, sBuff, *plLen);
	if((*plLen)%2 > 0)
	{
		// not word aligned, make it aligned
		*plLen = ((*plLen)/2)*2 + 2;
	}
	*plSysErr = RnbTkn_ReadFile(pContext, sBuff, 0, *plLen, &lActLen);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}
	*plLen = lActLen;
	return 0;
}

int WriteIKeyFile(PRNBTKN_CONTEXT pContext, unsigned char *sBuff, unsigned long *plLen, unsigned long *plSysErr, char *sMsg)
{
	unsigned long lActLen;
	unsigned char sAlignBuff[256];
	memset(sAlignBuff, 0, 256);
	memcpy(sAlignBuff, sBuff, *plLen);
	if((*plLen)%2 > 0)
	{
		// not word aligned, make it aligned
		*plLen = ((*plLen)/2)*2 + 2;
	}
	*plSysErr = RnbTkn_WriteFile(pContext, sBuff, 0, *plLen, &lActLen);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}
	*plLen = lActLen;
	return 0;
}

int DelIKeyDir(PRNBTKN_CONTEXT pContext, unsigned long lDir, unsigned long *plSysErr, char *sMsg)
{
	*plSysErr = RnbTkn_DeleteDir(pContext, lDir);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}
	return 0;
}

int DelIKeyFile(PRNBTKN_CONTEXT pContext, unsigned long lDir, unsigned long lFile, unsigned long *plSysErr, char *sMsg)
{
	*plSysErr = RnbTkn_DeleteFile(pContext, lDir, lFile);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}
	return 0;
}

int CreateIKeyDir(PRNBTKN_CONTEXT pContext, unsigned long lDir, unsigned long lLen, unsigned long *plSysErr, char *sMsg)
{
	if(lLen%8 > 0)
	{
		// not word aligned, make it aligned
		lLen = lLen/8 + 8;
	}
	*plSysErr = RnbTkn_CreateDir(pContext, lDir, lLen);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}
	return 0;
}


int CreateIKeyFile(PRNBTKN_CONTEXT pContext, 
				   unsigned long lDir, 
				   unsigned long lFile, 
				   unsigned long lLen, 
				   unsigned char cFileType,
				   unsigned char cReadAccess,
				   unsigned char cWriteAccess,
				   unsigned char cCryptAccess,
				   unsigned long *plSysErr, char *sMsg)
{
	RNBTKN_FILEINFO stFileInfo;

	stFileInfo.DirId = lDir;
	stFileInfo.Id = lFile;
	stFileInfo.Size = lLen;
	stFileInfo.Type = cFileType;

	stFileInfo.ReadAccess   = RNBTKN_ACCESS_ALWAYS;
	stFileInfo.WriteAccess  = RNBTKN_ACCESS_ALWAYS;
	stFileInfo.CryptAccess  = RNBTKN_ACCESS_ALWAYS;

	if( (RNBTKN_FILETYPE_DATA == stFileInfo.Type) || 
		(RNBTKN_FILETYPE_COUNTER == stFileInfo.Type) )
	{
		stFileInfo.ReadAccess = cReadAccess;
		stFileInfo.WriteAccess = cWriteAccess;
	}

	if(RNBTKN_FILETYPE_KEY == stFileInfo.Type)
	{
		stFileInfo.CryptAccess = cCryptAccess;
	}

    *plSysErr = RnbTkn_CreateFile(pContext, &stFileInfo);
	if(RB_SUCCESS != *plSysErr)
	{
		IKeyErrToMessage(*plSysErr, sMsg);
		return -1;
	}
	return 0;
}
