/*
 * @(#)IKMessageTool.cpp	1.00 10th June 2000
 * 
 * Functions that take care of translating error codes into messages.
 *
 * Modification Log:
 */

/**
 *
 * IKTool is a sample application using iKey, an user identity token from
 * Rainbow Technologies (http://www.rainbow.com).
 * Using this application you can personalize an iKey, and store data either
 * protected with a pin number or without protection, as you desire.
 * 
 * Several generic functions have been written for iKey, so that they
 * can be used with any type of application. 
 * 
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 10th June, 2000
 */

#include <stdio.h>
#include <string.h>
#include "IKMessageTool.hxx"

void IKeyPrintInfo(PRNBTKN_CONTEXT pContext)
{
	printf( "Firmware Version = %d.%d\n"
			"Product Code = %X\n"
			"Memory Size = %d (bytes)\n"
			"Serial No = %08lX%08lX\n",
			"Filesystem header size = %d (bytes)\n",
			pContext->ver_major, pContext->ver_minor,
			pContext->prod_code,
			pContext->mem_size,
			pContext->header_size,
			pContext->SerialNumber[1], pContext->SerialNumber[0] );

	printf("Capabilities: ");
	if(RNBTKN_CAPS_MD5HASH & pContext->capabilities) printf("MD5 Hash, ");
	if(RNBTKN_CAPS_MODEXP & pContext->capabilities) printf("MODEXP, ");
	if(RNBTKN_CAPS_DES & pContext->capabilities) printf("DES, ");
	if(RNBTKN_CAPS_ECC & pContext->capabilities) printf("ECC, ");
	if(RNBTKN_CAPS_MD5CHAP & pContext->capabilities) printf("MD5 CHAP, ");
	printf("\n");

	unsigned long ulStatus;
	RNBTKN_DIAGINFO diaginfo;

	if (RB_SUCCESS == (ulStatus = RnbTkn_Diagnose(pContext, &diaginfo)))
	{
		printf( "F/w Check-Sum = %04X", diaginfo.CheckSum);
	}
	else
	{
		char sMsg[1024];
		IKeyErrToMessage(ulStatus, sMsg);
		printf("Error diagnosing. %s\n");
	}
}

void IKeyErrToMessage(unsigned long ulStatus, char *sMsg)
{
    switch(ulStatus)
    {
        case RB_SUCCESS: 
			sprintf(sMsg, "Success. No Error."); 
			break;
        case RB_CANNOT_OPEN_DRIVER:
			sprintf(sMsg, "Cannot open driver. Driver not loaded? iKey is not detectable through the USB port. Is your OS configured for USB?"); 
			break;
        case RB_INVALID_DRVR_VERSION: 
			sprintf(sMsg, "Incompatible driver version. Please install the version required by this application."); 
			break;
        case RB_INVALID_COMMAND: 
			sprintf(sMsg, "Invalid command. Are you using older version of keys?"); 
			break;
        case RB_ACCESS_DENIED: 
			sprintf(sMsg, "Access Denied."); 
			break;
        case RB_ALREADY_ZERO: 
			sprintf(sMsg, "Counter value is already zero and cannot be decremented further."); 
			break;
        case RB_UNIT_NOT_FOUND: 
			sprintf(sMsg, "iKey not found. Please insert iKey into the USB port."); 
			break;
        case RB_DEVICE_REMOVED: 
			sprintf(sMsg, "iKey was removed. Please insert iKey into the USB port."); 
			break;
        case RB_COMMUNICATIONS_ERROR: 
			sprintf(sMsg, "Invalid response from iKey. Please retry again. If problem persists, reinstall drivers and reboot your machine."); 
			break;
        case RB_DIR_NOT_FOUND: 
			sprintf(sMsg, "The requested directory was not found."); 
			break;
        case RB_FILE_NOT_FOUND: 
			sprintf(sMsg, "The requested file was not found."); 
			break;
        case RB_MEM_CORRUPT: 
			sprintf(sMsg, "Contents of iKey memory are corrupted. The key may be defective or may need reloading."); 
			break;
        case RB_INTERNAL_HW_ERROR: 
			sprintf(sMsg, "Internal hardware error. The key may be defective or may need reloading."); 
			break;
        case RB_INVALID_RESP_SIZE: 
			sprintf(sMsg, "Invalid response size from iKey. Please retry again. If problem persists, reinstall drivers and reboot your machine."); 
			break;
        case RB_PIN_EXPIRED: 
			sprintf(sMsg, "PIN has expired and no more retries are allowed."); 
			break;
        case RB_ALREADY_EXISTS: 
			sprintf(sMsg, "File or directory already exists."); 
			break;
        case RB_NOT_ENOUGH_MEMORY:
			sprintf(sMsg, "Not enough memory available in the directory."); 
			break;
        case RB_INVALID_PARAMETER: 
			sprintf(sMsg, "Invalid parameter."); 
			break;
        case RB_INPUT_TOO_LONG: 
			sprintf(sMsg, "Input data longer than expected."); 
			break;
        case RB_ALIGNMENT_ERROR: 
			sprintf(sMsg, "Alignment Error. iKey internally must have data provided on word boundaries. Adjust alignment of supplied parameter."); 
			break;
        case RB_INVALID_STATUS: 
			sprintf(sMsg, "Unknown status returned from iKey. Check if iKey version is compatible."); 
			break;
        case RB_INVALID_FILE_SELECTED : 
			sprintf(sMsg, "Invalid file selected. File type or content is not valid."); 
			break;
        case RB_DEVICE_IN_USE: 
			sprintf(sMsg, "The device is in use by another application."); 
			break;
        default: 
			sprintf(sMsg, "Unknown Error [%04X]", ulStatus); 
			break;
    }
}
