/***************************************************************************
*
* (C) Copyright 1998-1999 Rainbow Technologies, Inc. All rights reserved.
*
* Description:  i-Key interface and definitions
*
* Environment:  User mode
*
* Revision History:
*
*   02/03/99: Mehdi - non-exportable functions made optional
*   12/12/98: Mehdi - created
* 
****************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

#define RNBTKN_API_VERSION          2

#define RNBTKN_API                  __stdcall

//
// Status return values
//

#define RB_SUCCESS                  0x00
#define RB_CANNOT_OPEN_DRIVER       0x01
#define RB_INVALID_DRVR_VERSION     0x02
#define RB_INVALID_COMMAND          0x03
#define RB_ACCESS_DENIED            0x04
#define RB_ALREADY_ZERO             0x05
#define RB_UNIT_NOT_FOUND           0x06
#define RB_DEVICE_REMOVED           0x07
#define RB_COMMUNICATIONS_ERROR     0x08
#define RB_DIR_NOT_FOUND            0x09
#define RB_FILE_NOT_FOUND           0x0A
#define RB_MEM_CORRUPT              0x0B
#define RB_INTERNAL_HW_ERROR        0x0C
#define RB_INVALID_RESP_SIZE        0x0D
#define RB_PIN_EXPIRED              0x0E
#define RB_ALREADY_EXISTS           0x0F
#define RB_NOT_ENOUGH_MEMORY        0x10
#define RB_INVALID_PARAMETER        0x11
#define RB_ALIGNMENT_ERROR          0x12
#define RB_INPUT_TOO_LONG           0x13
#define RB_INVALID_FILE_SELECTED    0x14
#define RB_DEVICE_IN_USE            0x15
#define RB_INVALID_STATUS           0xff

// Flags used by RnbTkn_OpenDevice()

#define RNBTKN_OPEN_NEXT            0x00000000
#define RNBTKN_OPEN_FIRST           0x00000001
#define RNBTKN_OPEN_CURRENT         0x00000002

// Flags in RnbTkn_HashToken()

#define RNBTKN_HASH_MD5_XOR         0x00
#define RNBTKN_HASH_MD5_HMAC        0x01
#define RNBTKN_HASH_MD5_CHAP        0x02

// RNBTKN_FILEINFO.Type

#define RNBTKN_FILETYPE_UNUSED      0x00
#define RNBTKN_FILETYPE_DIR         0x01
#define RNBTKN_FILETYPE_DATA        0x02
#define RNBTKN_FILETYPE_COUNTER     0x03
#define RNBTKN_FILETYPE_KEY         0x04
#define RNBTKN_FILETYPE_UNKNOWN     0xFF

// RNBTKN_FILEINFO access types

#define RNBTKN_ACCESS_ALWAYS        0x00
#define RNBTKN_ACCESS_PIN           0x01
#define RNBTKN_ACCESS_MASTERKEY     0x02
#define RNBTKN_ACCESS_NEVER         0x07

// RNBTKN_CONTEXT.FileAccess definition

#define RNBTKN_ACCESS_READ          0x01
#define RNBTKN_ACCESS_WRITE         0x02
#define RNBTKN_ACCESS_CRYPT         0x04

// LED state controls

#define RNBTKN_LED_SYSPOWER         0x00
#define RNBTKN_LED_ON               0x01
#define RNBTKN_LED_OFF              0x02

typedef struct
{
    unsigned long   Magic;
    unsigned long   DeviceHandle;
    unsigned long   ClientHandle;
    unsigned long   Flags;
    unsigned long   library_version;
    unsigned long   driver_version;

    // Firmware/hardware information

    unsigned char   ver_major;
    unsigned char   ver_minor;
    unsigned char   prod_code;
    unsigned char   config;
    unsigned long   header_size;
    unsigned long   modulus_size;
    unsigned long   mem_size;
    unsigned long   capabilities;
    unsigned long   SerialNumber[2];

    // Selected file information: updated by RnbTkn_SelectFile()

    unsigned long   DirId;
    unsigned long   FileId;
    unsigned long   FileSize;
    unsigned char   FileType;
    unsigned char   FileAccess;         // granted access
    unsigned char   Padding1[2];

    unsigned char   Reserved[356];

} RNBTKN_CONTEXT, *PRNBTKN_CONTEXT;

// RNBTKN_CONTEXT.capabilities definitions

#define RNBTKN_CAPS_MD5HASH     0x00000001
#define RNBTKN_CAPS_MODEXP      0x00000002
#define RNBTKN_CAPS_DES         0x00000004
#define RNBTKN_CAPS_ECC         0x00000008
#define RNBTKN_CAPS_MD5CHAP     0x00000010

typedef struct
{
    unsigned long   Id;
    unsigned long   DirId;
    unsigned long   Size;
    unsigned char   Type;
    unsigned char   ReadAccess;
    unsigned char   WriteAccess;
    unsigned char   CryptAccess;

} RNBTKN_FILEINFO, *PRNBTKN_FILEINFO;

typedef struct
{
    unsigned char   MaxPinRetries;
    unsigned char   CurPinCounter;
    unsigned char   CreateAccess;
    unsigned char   DeleteAccess;

} RNBTKN_ACCESSINFO, *PRNBTKN_ACCESSINFO;

typedef struct
{
    unsigned short  CheckSum;
    unsigned short  HwInfo;

} RNBTKN_DIAGINFO, *PRNBTKN_DIAGINFO;


/////////////////////////////////////////////////////////////////////////////
//
// Normal device operations
//
/////////////////////////////////////////////////////////////////////////////

unsigned long RNBTKN_API
RnbTkn_OpenDevice       ( PRNBTKN_CONTEXT ctx,
                          unsigned long   Flags         );

unsigned long RNBTKN_API
RnbTkn_CloseDevice      ( PRNBTKN_CONTEXT ctx           );

unsigned long RNBTKN_API
RnbTkn_ResetDevice      ( PRNBTKN_CONTEXT ctx           );

unsigned long RNBTKN_API
RnbTkn_GetSerialNumber  ( PRNBTKN_CONTEXT ctx,
                          unsigned long * SerialNumber  );

unsigned long RNBTKN_API
RnbTkn_Dir              ( PRNBTKN_CONTEXT ctx,
                          unsigned long   Dir_Id,
                          unsigned long   Index,
                          PRNBTKN_FILEINFO FileInfo     );

unsigned long RNBTKN_API
RnbTkn_SelectFile       ( PRNBTKN_CONTEXT ctx,
                          unsigned long   Dir_Id,
                          unsigned long   File_Id       );

unsigned long RNBTKN_API
RnbTkn_CloseFile        ( PRNBTKN_CONTEXT ctx           );

unsigned long RNBTKN_API
RnbTkn_ReadFile         ( PRNBTKN_CONTEXT ctx,
                          unsigned char * Buffer,
                          unsigned long   Offset,
                          unsigned long   Length,
                          unsigned long * nReturned     );

unsigned long RNBTKN_API
RnbTkn_WriteFile        ( PRNBTKN_CONTEXT ctx,
                          unsigned char * Buffer,
                          unsigned long   Offset,
                          unsigned long   Length,
                          unsigned long * nWritten      );

unsigned long RNBTKN_API
RnbTkn_Decrement        ( PRNBTKN_CONTEXT ctx           );

unsigned long RNBTKN_API
RnbTkn_HashToken        ( PRNBTKN_CONTEXT ctx,
                          unsigned long   Flags,
                          unsigned char * text,
                          unsigned long   len,
                          unsigned char * Digest        );

unsigned long RNBTKN_API
RnbTkn_HMAC_MD5         ( PRNBTKN_CONTEXT ctx,
                          unsigned long   key1_dir,
                          unsigned long   key1_id,
                          unsigned long   key2_dir,
                          unsigned long   key2_id,
                          unsigned char * text,
                          int             text_len,
                          unsigned char * digest        );

unsigned long RNBTKN_API
RnbTkn_CHAP_MD5         ( PRNBTKN_CONTEXT ctx,
                          unsigned long   key_dir,
                          unsigned long   key_id,
                          unsigned char   identifier,
                          unsigned char*  text,
                          int             text_len,
                          unsigned char*  digest        );

unsigned long RNBTKN_API
RnbTkn_VerifyPIN        ( PRNBTKN_CONTEXT ctx,
                          unsigned long   PIN           );

unsigned long RNBTKN_API
RnbTkn_GetChallenge     ( PRNBTKN_CONTEXT ctx,
                          unsigned char * challenge     );


/////////////////////////////////////////////////////////////////////////////
//
// Device management functions
//
/////////////////////////////////////////////////////////////////////////////

unsigned long RNBTKN_API
RnbTkn_CreateDir        ( PRNBTKN_CONTEXT ctx,
                          unsigned long   Dir_Id,
                          unsigned long   Length        );

unsigned long RNBTKN_API
RnbTkn_CreateFile       ( PRNBTKN_CONTEXT ctx,
                          PRNBTKN_FILEINFO FileInfo     );

unsigned long RNBTKN_API
RnbTkn_DeleteDir        ( PRNBTKN_CONTEXT ctx,
                          unsigned long   Dir_Id        );

unsigned long RNBTKN_API
RnbTkn_DeleteFile       ( PRNBTKN_CONTEXT ctx,
                          unsigned long   Dir_Id,
                          unsigned long   File_Id       );

unsigned long RNBTKN_API
RnbTkn_DeleteAllFiles   ( PRNBTKN_CONTEXT ctx           );

unsigned long RNBTKN_API
RnbTkn_SetAccessSettings( PRNBTKN_CONTEXT ctx,
                          PRNBTKN_ACCESSINFO accessInfo );

unsigned long RNBTKN_API
RnbTkn_ModifyPIN        ( PRNBTKN_CONTEXT ctx,
                          unsigned long   Pin           );

unsigned long RNBTKN_API
RnbTkn_ModifyMasterKey  ( PRNBTKN_CONTEXT ctx,
                          unsigned char * MasterKey     );

unsigned long RNBTKN_API
RnbTkn_VerifyMasterKey  ( PRNBTKN_CONTEXT ctx,
                          unsigned char * MasterKey     );


/////////////////////////////////////////////////////////////////////////////
//
// Misc. functions
//
/////////////////////////////////////////////////////////////////////////////

unsigned long RNBTKN_API
RnbTkn_GetAccessSettings( PRNBTKN_CONTEXT ctx,
                          PRNBTKN_ACCESSINFO accessInfo );

unsigned long RNBTKN_API
RnbTkn_LedControl       ( PRNBTKN_CONTEXT ctx,
                          unsigned long   LedState      );

unsigned long RNBTKN_API
RnbTkn_Diagnose         ( PRNBTKN_CONTEXT ctx,
                          PRNBTKN_DIAGINFO diaginfo     );

#ifdef __cplusplus
}
#endif

