/***************************************************************************
*
* (C) Copyright 1998-1999 Rainbow Technologies, Inc. All rights reserved.
*
* Description:  MD5 support
*
* Environment:  User mode
*
* Revision History:
*
*   12/12/98: Mehdi - created
* 
****************************************************************************/

#include "md5.h"
#include "tokenmd5.h"

unsigned char *
__stdcall
MD5_Hash( unsigned char *Message,
          unsigned long Length,
          unsigned char *Digest )
{
    MD5_CTX context;

    MD5Init( &context );
    MD5Update( &context, Message, Length );
    MD5Final( Digest, &context );

    return Digest;
}

unsigned char*
__stdcall
HMAC_MD5( unsigned char*  text,         /* pointer to data stream        */
          int             text_len,     /* length of data stream         */
          unsigned char*  key,          /* pointer to authentication key */
          int             key_len,      /* length of authentication key  */
          unsigned char*  key1,         /* inner hashed key1             */
          unsigned char*  key2,         /* inner hashed key2             */
          unsigned char*  digest )      /* caller digest to be filled in */
{
    MD5_CTX context;
    unsigned char k_ipad[64];    /* inner padding key XORd with ipad */
    unsigned char k_opad[64];    /* outer padding key XORd with opad */
    unsigned char tk[16];
    int i;

    /* if key is longer than 64 bytes reset it to key=MD5(key) */
    if (key_len > 64)
    {
        MD5_Hash(key, key_len,tk);
        key = tk;
        key_len = 16;
    }

    /*
     * the HMAC_MD5 transform looks like:
     *
     * MD5(K XOR opad, MD5(K XOR ipad, text))
     *
     * where K is an n byte key
     * ipad is the byte 0x36 repeated 64 times
     * opad is the byte 0x5c repeated 64 times
     * and text is the data being protected
     */

    for( i = 0; i < key_len; i++ )
    {
        k_ipad[i] = key[i] ^ 0x36;
        k_opad[i] = key[i] ^ 0x5c;
    }

    while( i < 64 )
    {
        k_ipad[i] = 0x36;
        k_opad[i] = 0x5c;
        i++;
    }

    // perform inner MD5
    MD5Init(&context);                   /* init context for 1st pass */
    MD5Update(&context, k_ipad, 64);     /* start with inner pad */

    // KEY1 is the current state of the hash
    if( key1 ) for (i = 0; i < 4; i++ )
    {
        *key1++ = (unsigned char)((context.buf[i]      ) & 0xFF);
        *key1++ = (unsigned char)((context.buf[i] >>  8) & 0xFF);
        *key1++ = (unsigned char)((context.buf[i] >> 16) & 0xFF);
        *key1++ = (unsigned char)((context.buf[i] >> 24) & 0xFF);
    }

    MD5Update(&context, text, text_len); /* then text of datagram */
    MD5Final(digest, &context);          /* finish up 1st pass */

    // perform outer MD5
    MD5Init(&context);                   /* init context for 2nd pass */
    MD5Update(&context, k_opad, 64);     /* start with outer pad */

    if( key2 ) for (i = 0; i < 4; i++ )
    {
        *key2++ = (unsigned char)((context.buf[i]      ) & 0xFF);
        *key2++ = (unsigned char)((context.buf[i] >>  8) & 0xFF);
        *key2++ = (unsigned char)((context.buf[i] >> 16) & 0xFF);
        *key2++ = (unsigned char)((context.buf[i] >> 24) & 0xFF);
    }

    MD5Update(&context, digest, 16);     /* then results of 1st hash */
    MD5Final(digest, &context);          /* finish up 2nd pass */

    return digest;
}

unsigned char*
__stdcall
CHAP_MD5( unsigned char   identifier,
          unsigned char*  secret,
          int             secret_len,
          unsigned char*  challenge,
          int             challenge_len,
          unsigned char*  digest )
{
    MD5_CTX context;

    MD5Init( &context );

    MD5Update( &context, &identifier, 1 );
    MD5Update( &context, secret, secret_len );
    MD5Update( &context, challenge, challenge_len );
    MD5Final( digest, &context );

    return digest;
}

